#include "../subghz_i.h"
#include "../helpers/subghz_custom_event.h"
#include <dolphin/dolphin.h>

void subghz_scene_receiver_info_callback(GuiButtonType result, InputType type, void* context) {
    furi_assert(context);
    SubGhz* subghz = context;

    if((result == GuiButtonTypeCenter) && (type == InputTypePress)) {
        view_dispatcher_send_custom_event(
            subghz->view_dispatcher, SubGhzCustomEventSceneReceiverInfoTxStart);
    } else if((result == GuiButtonTypeCenter) && (type == InputTypeRelease)) {
        view_dispatcher_send_custom_event(
            subghz->view_dispatcher, SubGhzCustomEventSceneReceiverInfoTxStop);
    } else if((result == GuiButtonTypeRight) && (type == InputTypeShort)) {
        view_dispatcher_send_custom_event(
            subghz->view_dispatcher, SubGhzCustomEventSceneReceiverInfoSave);
    }
}

static bool subghz_scene_receiver_info_update_parser(void* context) {
    SubGhz* subghz = context;
    subghz->txrx->decoder_result = subghz_receiver_search_decoder_base_by_name(
        subghz->txrx->receiver,
        subghz_history_get_protocol_name(subghz->txrx->history, subghz->txrx->idx_menu_chosen));
    if(subghz->txrx->decoder_result) {
        subghz_protocol_decoder_base_deserialize(
            subghz->txrx->decoder_result,
            subghz_history_get_raw_data(subghz->txrx->history, subghz->txrx->idx_menu_chosen));
        subghz->txrx->frequency =
            subghz_history_get_frequency(subghz->txrx->history, subghz->txrx->idx_menu_chosen);
        subghz->txrx->preset =
            subghz_history_get_preset(subghz->txrx->history, subghz->txrx->idx_menu_chosen);
        return true;
    }
    return false;
}

void subghz_scene_receiver_info_on_enter(void* context) {
    SubGhz* subghz = context;

    DOLPHIN_DEED(DolphinDeedSubGhzReceiverInfo);
    if(subghz_scene_receiver_info_update_parser(subghz)) {
        string_t frequency_str;
        string_t modulation_str;
        string_t text;

        string_init(frequency_str);
        string_init(modulation_str);
        string_init(text);

        subghz_get_frequency_modulation(subghz, frequency_str, modulation_str);
        widget_add_string_element(
            subghz->widget,
            78,
            0,
            AlignLeft,
            AlignTop,
            FontSecondary,
            string_get_cstr(frequency_str));

        widget_add_string_element(
            subghz->widget,
            113,
            0,
            AlignLeft,
            AlignTop,
            FontSecondary,
            string_get_cstr(modulation_str));
        subghz_protocol_decoder_base_get_string(subghz->txrx->decoder_result, text);
        widget_add_string_multiline_element(
            subghz->widget, 0, 0, AlignLeft, AlignTop, FontSecondary, string_get_cstr(text));

        string_clear(frequency_str);
        string_clear(modulation_str);
        string_clear(text);

        if((subghz->txrx->decoder_result->protocol->flag & SubGhzProtocolFlag_Save) ==
           SubGhzProtocolFlag_Save) {
            widget_add_button_element(
                subghz->widget,
                GuiButtonTypeRight,
                "Save",
                subghz_scene_receiver_info_callback,
                subghz);
        }
        if(((subghz->txrx->decoder_result->protocol->flag & SubGhzProtocolFlag_Send) ==
            SubGhzProtocolFlag_Send) &&
           subghz->txrx->decoder_result->protocol->encoder->deserialize &&
           subghz->txrx->decoder_result->protocol->type == SubGhzProtocolTypeStatic) {
            widget_add_button_element(
                subghz->widget,
                GuiButtonTypeCenter,
                "Send",
                subghz_scene_receiver_info_callback,
                subghz);
        }
    } else {
        widget_add_icon_element(subghz->widget, 32, 12, &I_DolphinFirstStart7_61x51);
        widget_add_string_element(
            subghz->widget, 13, 8, AlignLeft, AlignBottom, FontSecondary, "Error history parse.");
    }

    view_dispatcher_switch_to_view(subghz->view_dispatcher, SubGhzViewIdWidget);
}

bool subghz_scene_receiver_info_on_event(void* context, SceneManagerEvent event) {
    SubGhz* subghz = context;
    if(event.type == SceneManagerEventTypeCustom) {
        if(event.event == SubGhzCustomEventSceneReceiverInfoTxStart) {
            //CC1101 Stop RX -> Start TX
            if(subghz->txrx->hopper_state != SubGhzHopperStateOFF) {
                subghz->txrx->hopper_state = SubGhzHopperStatePause;
            }
            if(subghz->txrx->txrx_state == SubGhzTxRxStateRx) {
                subghz_rx_end(subghz);
            }
            if(!subghz_scene_receiver_info_update_parser(subghz)) {
                return false;
            }
            if(subghz->txrx->txrx_state == SubGhzTxRxStateIDLE ||
               subghz->txrx->txrx_state == SubGhzTxRxStateSleep) {
                if(!subghz_tx_start(
                       subghz,
                       subghz_history_get_raw_data(
                           subghz->txrx->history, subghz->txrx->idx_menu_chosen))) {
                    scene_manager_next_scene(subghz->scene_manager, SubGhzSceneShowOnlyRx);
                } else {
                    subghz->state_notifications = SubGhzNotificationStateTX;
                }
            }
            return true;
        } else if(event.event == SubGhzCustomEventSceneReceiverInfoTxStop) {
            //CC1101 Stop Tx -> Start RX
            subghz->state_notifications = SubGhzNotificationStateIDLE;
            if(subghz->txrx->txrx_state == SubGhzTxRxStateTx) {
                subghz_tx_stop(subghz);
            }
            if(subghz->txrx->txrx_state == SubGhzTxRxStateIDLE) {
                subghz_begin(subghz, subghz->txrx->preset);
                subghz_rx(subghz, subghz->txrx->frequency);
            }
            if(subghz->txrx->hopper_state == SubGhzHopperStatePause) {
                subghz->txrx->hopper_state = SubGhzHopperStateRunnig;
            }
            subghz->state_notifications = SubGhzNotificationStateRX;
            return true;
        } else if(event.event == SubGhzCustomEventSceneReceiverInfoSave) {
            //CC1101 Stop RX -> Save
            subghz->state_notifications = SubGhzNotificationStateIDLE;
            if(subghz->txrx->hopper_state != SubGhzHopperStateOFF) {
                subghz->txrx->hopper_state = SubGhzHopperStateOFF;
            }
            if(subghz->txrx->txrx_state == SubGhzTxRxStateRx) {
                subghz_rx_end(subghz);
                subghz_sleep(subghz);
            }
            if(!subghz_scene_receiver_info_update_parser(subghz)) {
                return false;
            }

            if((subghz->txrx->decoder_result->protocol->flag & SubGhzProtocolFlag_Save) ==
               SubGhzProtocolFlag_Save) {
                subghz_file_name_clear(subghz);
                scene_manager_next_scene(subghz->scene_manager, SubGhzSceneSaveName);
            }
            return true;
        }
    } else if(event.type == SceneManagerEventTypeTick) {
        if(subghz->txrx->hopper_state != SubGhzHopperStateOFF) {
            subghz_hopper_update(subghz);
        }
        switch(subghz->state_notifications) {
        case SubGhzNotificationStateTX:
            notification_message(subghz->notifications, &sequence_blink_red_10);
            break;
        case SubGhzNotificationStateRX:
            notification_message(subghz->notifications, &sequence_blink_blue_10);
            break;
        default:
            break;
        }
    }
    return false;
}

void subghz_scene_receiver_info_on_exit(void* context) {
    SubGhz* subghz = context;
    widget_reset(subghz->widget);
}
